==> cached_properties-0.6.tar.gz: cached_properties-0.6/cached_properties/properties.py <==
from time import time

__all__ = ['Property']


class Property(object):
	def __init__(self, method=None, *, general=None, timeout=None, hits=None):
		self.general = general
		self.timeout = timeout
		self.hits = hits
		if method is not None:
			self._setup_method(method)

	def __set__(self, instance, value):
		self.storage = self.__dict__ if self.general else instance.__dict__
		if self._var in self.storage:
			storage.__dict__[self._var]['value'] = value
		else:
			self._update_copy({
				'value': value,
				'timeout': (time(), self.timeout),
				'hits': self.hits
				}
			)
		return self.storage[self._var]['value']

	def __call__(self, method):
		self._setup_method(method)
		return self

	def __get__(self, instance, cls):
		self.storage = self.__dict__ if self.general else instance.__dict__
		return self._update_storage(instance, cls)

	def __delete__(self, instance):
		self.storage = self.__dict__ if self.general else instance.__dict__
		storage.pop(self._var, None)

	def _setup_method(self, method):
		self.method = method
		self._var = f'_s_{method.__name__}'

	def _update_copy(self, value):
		const = {
			'value': None,  # self.method(instance),
			'timeout': None,  # (time(), self.timeout),
			'hits': self.hits}
		dicopy = const.copy()
		dicopy.update(value)
		self.storage[self._var] = dicopy
		return self.storage[self._var]['value']

	def _update_storage(self, instance, cls):
		if self._var not in self.storage:
			return self._update_copy({
				'value': self.method(instance),
				'timeout': (time(), self.timeout)})
		if self.hits:
			self.storage[self._var]['hits'] -= 1
			if self.storage[self._var]['hits'] <= 1:
				# make sure we don't change the old time when checking hits
				settime = self.storage[self._var]['timeout'][0]
				return self._update_copy({
					'value': self.method(instance),
					'timeout': (settime, self.timeout),
					'hits': self.hits})
		if self.timeout:
			settime, timeout = self.storage[self._var]['timeout']
			if time() - settime >= timeout:
				return self._update_storage({
					'value': self.method(instance),
					'timeout': (time(), self.timeout),
					'hits': self.hits})
		return self.storage[self._var]['value']

==> cached_properties-0.7.tar.gz: cached_properties-0.7/cached_properties/properties.py <==
from time import time

__all__ = ['Property']


class Property(object):
	def __init__(self, method=None, *, general=None, timeout=None, hits=None):
		self.general = general
		self.timeout = timeout
		self.hits = hits
		if method is not None:
			self._setup_method(method)

	def __set__(self, instance, value):
		self.storage = self.__dict__ if self.general else instance.__dict__
		if self._var in self.storage:
			self.storage.__dict__[self._var]['value'] = value
		else:
			self._update_copy({
				'value': value,
				'timeout': (time(), self.timeout),
				'hits': self.hits
				}
			)
		return self.storage[self._var]['value']

	def __call__(self, method):
		self._setup_method(method)
		return self

	def __get__(self, instance, cls):
		self.storage = self.__dict__ if self.general else instance.__dict__
		return self._update_storage(instance, cls)

	def __delete__(self, instance):
		self.storage = self.__dict__ if self.general else instance.__dict__
		storage.pop(self._var, None)

	def _setup_method(self, method):
		self.method = method
		self._var = f'_s_{method.__name__}'

	def _update_copy(self, value):
		const = {
			'value': None,  # self.method(instance),
			'timeout': None,  # (time(), self.timeout),
			'hits': self.hits}
		dicopy = const.copy()
		dicopy.update(value)
		self.storage[self._var] = dicopy
		return self.storage[self._var]['value']

	def _update_storage(self, instance, cls):
		if self._var not in self.storage:
			return self._update_copy({
				'value': self.method(instance),
				'timeout': (time(), self.timeout)})
		if self.hits:
			self.storage[self._var]['hits'] -= 1
			if self.storage[self._var]['hits'] <= 1:
				# make sure we don't change the old time when checking hits
				settime = self.storage[self._var]['timeout'][0]
				return self._update_copy({
					'value': self.method(instance),
					'timeout': (settime, self.timeout),
					'hits': self.hits})
		if self.timeout:
			settime, timeout = self.storage[self._var]['timeout']
			if time() - settime >= timeout:
				return self._update_storage({
					'value': self.method(instance),
					'timeout': (time(), self.timeout),
					'hits': self.hits})
		return self.storage[self._var]['value']
